@@ -31,31 +31,22 @@
 #include <errno.h>
 
 #include <libavcodec/avcodec.h>
+#include <libavutil/opt.h>
 
 #include <gst/gst.h>
 
 #include "gstav.h"
+#include "gstavcfg.h"
 #include "gstavcodecmap.h"
 #include "gstavutils.h"
 #include "gstavaudenc.h"
 
-#define DEFAULT_AUDIO_BITRATE 128000
-
-enum
-{
-  /* FILL ME */
-  LAST_SIGNAL
-};
-
 enum
 {
   PROP_0,
-  PROP_BIT_RATE,
-  PROP_RTP_PAYLOAD_SIZE,
-  PROP_COMPLIANCE,
+  PROP_CFG_BASE,
 };
 
-/* A number of function prototypes are given so we can refer to them later. */
 static void gst_ffmpegaudenc_class_init (GstFFMpegAudEncClass * klass);
 static void gst_ffmpegaudenc_base_init (GstFFMpegAudEncClass * klass);
 static void gst_ffmpegaudenc_init (GstFFMpegAudEnc * ffmpegaudenc);
@@ -78,8 +69,6 @@ static void gst_ffmpegaudenc_get_property (GObject * object,
 
 static GstElementClass *parent_class = NULL;
 
-/*static guint gst_ffmpegaudenc_signals[LAST_SIGNAL] = { 0 }; */
-
 static void
 gst_ffmpegaudenc_base_init (GstFFMpegAudEncClass * klass)
 {
@@ -148,16 +137,8 @@ gst_ffmpegaudenc_class_init (GstFFMpegAudEncClass * klass)
   gobject_class->set_property = gst_ffmpegaudenc_set_property;
   gobject_class->get_property = gst_ffmpegaudenc_get_property;
 
-  /* FIXME: could use -1 for a sensible per-codec defaults */
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_BIT_RATE,
-      g_param_spec_int ("bitrate", "Bit Rate",
-          "Target Audio Bitrate", 0, G_MAXINT, DEFAULT_AUDIO_BITRATE,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_COMPLIANCE,
-      g_param_spec_enum ("compliance", "Compliance",
-          "Adherence of the encoder to the specifications",
-          GST_TYPE_FFMPEG_COMPLIANCE, FFMPEG_DEFAULT_COMPLIANCE,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+  gst_ffmpeg_cfg_install_properties (gobject_class, klass->in_plugin,
+      PROP_CFG_BASE, AV_OPT_FLAG_ENCODING_PARAM | AV_OPT_FLAG_AUDIO_PARAM);
 
   gobject_class->finalize = gst_ffmpegaudenc_finalize;
 
@@ -180,11 +161,10 @@ gst_ffmpegaudenc_init (GstFFMpegAudEnc * ffmpegaudenc)
 
   /* ffmpeg objects */
   ffmpegaudenc->context = avcodec_alloc_context3 (klass->in_plugin);
+  ffmpegaudenc->refcontext = avcodec_alloc_context3 (klass->in_plugin);
   ffmpegaudenc->opened = FALSE;
   ffmpegaudenc->frame = av_frame_alloc ();
 
-  ffmpegaudenc->compliance = FFMPEG_DEFAULT_COMPLIANCE;
-
   gst_audio_encoder_set_drainable (GST_AUDIO_ENCODER (ffmpegaudenc), TRUE);
 }
 
@@ -196,7 +176,9 @@ gst_ffmpegaudenc_finalize (GObject * object)
   /* clean up remaining allocated data */
   av_frame_free (&ffmpegaudenc->frame);
   gst_ffmpeg_avcodec_close (ffmpegaudenc->context);
+  gst_ffmpeg_avcodec_close (ffmpegaudenc->refcontext);
   av_free (ffmpegaudenc->context);
+  av_free (ffmpegaudenc->refcontext);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -262,32 +244,7 @@ gst_ffmpegaudenc_set_format (GstAudioEncoder * encoder, GstAudioInfo * info)
     }
   }
 
-  /* if we set it in _getcaps we should set it also in _link */
-  ffmpegaudenc->context->strict_std_compliance = ffmpegaudenc->compliance;
-
-  /* user defined properties */
-  if (ffmpegaudenc->bitrate > 0) {
-    GST_INFO_OBJECT (ffmpegaudenc, "Setting avcontext to bitrate %d",
-        ffmpegaudenc->bitrate);
-    ffmpegaudenc->context->bit_rate = ffmpegaudenc->bitrate;
-    ffmpegaudenc->context->bit_rate_tolerance = ffmpegaudenc->bitrate;
-  } else {
-    GST_INFO_OBJECT (ffmpegaudenc,
-        "Using avcontext default bitrate %" G_GINT64_FORMAT,
-        (gint64) ffmpegaudenc->context->bit_rate);
-  }
-
-  /* RTP payload used for GOB production (for Asterisk) */
-  if (ffmpegaudenc->rtp_payload_size) {
-    ffmpegaudenc->context->rtp_payload_size = ffmpegaudenc->rtp_payload_size;
-  }
-
-  /* some other defaults */
-  ffmpegaudenc->context->rc_strategy = 2;
-  ffmpegaudenc->context->b_frame_strategy = 0;
-  ffmpegaudenc->context->coder_type = 0;
-  ffmpegaudenc->context->context_model = 0;
-  ffmpegaudenc->context->scenechange_threshold = 0;
+  gst_ffmpeg_cfg_fill_context (G_OBJECT (ffmpegaudenc), ffmpegaudenc->context);
 
   /* fetch pix_fmt and so on */
   gst_ffmpeg_audioinfo_to_context (info, ffmpegaudenc->context);
@@ -330,8 +287,9 @@ gst_ffmpegaudenc_set_format (GstAudioEncoder * encoder, GstAudioInfo * info)
             oclass->in_plugin) < 0)
       GST_DEBUG_OBJECT (ffmpegaudenc, "Failed to set context defaults");
 
-    if ((oclass->in_plugin->capabilities & CODEC_CAP_EXPERIMENTAL) &&
-        ffmpegaudenc->compliance != GST_FFMPEG_EXPERIMENTAL) {
+    if ((oclass->in_plugin->capabilities & AV_CODEC_CAP_EXPERIMENTAL) &&
+        ffmpegaudenc->context->strict_std_compliance !=
+        FF_COMPLIANCE_EXPERIMENTAL) {
       GST_ELEMENT_ERROR (ffmpegaudenc, LIBRARY, SETTINGS,
           ("Codec is experimental, but settings don't allow encoders to "
               "produce output of experimental quality"),
@@ -445,15 +403,13 @@ buffer_info_free (void *opaque, guint8 * data)
 }
 
 static GstFlowReturn
-gst_ffmpegaudenc_encode_audio (GstFFMpegAudEnc * ffmpegaudenc,
-    GstBuffer * buffer, gint * have_data)
+gst_ffmpegaudenc_send_frame (GstFFMpegAudEnc * ffmpegaudenc, GstBuffer * buffer)
 {
   GstAudioEncoder *enc;
   AVCodecContext *ctx;
-  gint res;
   GstFlowReturn ret;
+  gint res;
   GstAudioInfo *info;
-  AVPacket *pkt;
   AVFrame *frame = ffmpegaudenc->frame;
   gboolean planar;
   gint nsamples = -1;
@@ -462,8 +418,6 @@ gst_ffmpegaudenc_encode_audio (GstFFMpegAudEnc * ffmpegaudenc,
 
   ctx = ffmpegaudenc->context;
 
-  pkt = g_slice_new0 (AVPacket);
-
   if (buffer != NULL) {
     BufferInfo *buffer_info = g_slice_new0 (BufferInfo);
     guint8 *audio_in;
@@ -572,28 +526,47 @@ gst_ffmpegaudenc_encode_audio (GstFFMpegAudEnc * ffmpegaudenc,
     }
 
     /* we have a frame to feed the encoder */
-    res = avcodec_encode_audio2 (ctx, pkt, frame, have_data);
+    res = avcodec_send_frame (ctx, frame);
 
     av_frame_unref (frame);
   } else {
     GST_LOG_OBJECT (ffmpegaudenc, "draining");
     /* flushing the encoder */
-    res = avcodec_encode_audio2 (ctx, pkt, NULL, have_data);
+    res = avcodec_send_frame (ctx, NULL);
   }
 
-  if (res < 0) {
-    char error_str[128] = { 0, };
-
-    g_slice_free (AVPacket, pkt);
-    av_strerror (res, error_str, sizeof (error_str));
-    GST_ERROR_OBJECT (enc, "Failed to encode buffer: %d - %s", res, error_str);
-    return GST_FLOW_OK;
+  if (res == 0) {
+    ret = GST_FLOW_OK;
+  } else if (res == AVERROR_EOF) {
+    ret = GST_FLOW_EOS;
+  } else {                      /* Any other return value is an error in our context */
+    ret = GST_FLOW_OK;
+    GST_WARNING_OBJECT (ffmpegaudenc, "Failed to encode buffer");
   }
-  GST_LOG_OBJECT (ffmpegaudenc, "got output size %d", res);
 
-  if (*have_data) {
+  return ret;
+}
+
+static GstFlowReturn
+gst_ffmpegaudenc_receive_packet (GstFFMpegAudEnc * ffmpegaudenc,
+    gboolean * got_packet)
+{
+  GstAudioEncoder *enc;
+  AVCodecContext *ctx;
+  gint res;
+  GstFlowReturn ret;
+  AVPacket *pkt;
+
+  enc = GST_AUDIO_ENCODER (ffmpegaudenc);
+
+  ctx = ffmpegaudenc->context;
+
+  pkt = g_slice_new0 (AVPacket);
+
+  res = avcodec_receive_packet (ctx, pkt);
+
+  if (res == 0) {
     GstBuffer *outbuf;
-    const AVCodec *codec;
 
     GST_LOG_OBJECT (ffmpegaudenc, "pushing size %d", pkt->size);
 
@@ -601,45 +574,39 @@ gst_ffmpegaudenc_encode_audio (GstFFMpegAudEnc * ffmpegaudenc,
         gst_buffer_new_wrapped_full (GST_MEMORY_FLAG_READONLY, pkt->data,
         pkt->size, 0, pkt->size, pkt, gst_ffmpegaudenc_free_avpacket);
 
-    codec = ffmpegaudenc->context->codec;
-    if ((codec->capabilities & CODEC_CAP_VARIABLE_FRAME_SIZE) || !buffer) {
-      /* FIXME: Not really correct, as -1 means "all the samples we got
-         given so far", which may not be true depending on the codec,
-         but we have no way to know AFAICT */
-      ret = gst_audio_encoder_finish_frame (enc, outbuf, -1);
-    } else {
-      ret = gst_audio_encoder_finish_frame (enc, outbuf, nsamples);
-    }
+    ret =
+        gst_audio_encoder_finish_frame (enc, outbuf,
+        pkt->duration > 0 ? pkt->duration : -1);
+    *got_packet = TRUE;
   } else {
     GST_LOG_OBJECT (ffmpegaudenc, "no output produced");
     g_slice_free (AVPacket, pkt);
     ret = GST_FLOW_OK;
+    *got_packet = FALSE;
   }
 
   return ret;
 }
 
-static void
+static GstFlowReturn
 gst_ffmpegaudenc_drain (GstFFMpegAudEnc * ffmpegaudenc)
 {
-  GstFFMpegAudEncClass *oclass;
+  GstFlowReturn ret = GST_FLOW_OK;
+  gboolean got_packet;
 
-  oclass = (GstFFMpegAudEncClass *) (G_OBJECT_GET_CLASS (ffmpegaudenc));
-
-  if (oclass->in_plugin->capabilities & CODEC_CAP_DELAY) {
-    gint have_data, try = 0;
-
-    GST_LOG_OBJECT (ffmpegaudenc,
-        "codec has delay capabilities, calling until libav has drained everything");
+  ret = gst_ffmpegaudenc_send_frame (ffmpegaudenc, NULL);
 
+  if (ret == GST_FLOW_OK) {
     do {
-      GstFlowReturn ret;
-
-      ret = gst_ffmpegaudenc_encode_audio (ffmpegaudenc, NULL, &have_data);
-      if (ret != GST_FLOW_OK || have_data == 0)
+      ret = gst_ffmpegaudenc_receive_packet (ffmpegaudenc, &got_packet);
+      if (ret != GST_FLOW_OK)
         break;
-    } while (try++ < 10);
+    } while (got_packet);
   }
+
+  avcodec_flush_buffers (ffmpegaudenc->context);
+
+  return ret;
 }
 
 static GstFlowReturn
@@ -647,17 +614,15 @@ gst_ffmpegaudenc_handle_frame (GstAudioEncoder * encoder, GstBuffer * inbuf)
 {
   GstFFMpegAudEnc *ffmpegaudenc;
   GstFlowReturn ret;
-  gint have_data;
+  gboolean got_packet;
 
   ffmpegaudenc = (GstFFMpegAudEnc *) encoder;
 
   if (G_UNLIKELY (!ffmpegaudenc->opened))
     goto not_negotiated;
 
-  if (!inbuf) {
-    gst_ffmpegaudenc_drain (ffmpegaudenc);
-    return GST_FLOW_OK;
-  }
+  if (!inbuf)
+    return gst_ffmpegaudenc_drain (ffmpegaudenc);
 
   inbuf = gst_buffer_ref (inbuf);
 
@@ -675,10 +640,14 @@ gst_ffmpegaudenc_handle_frame (GstAudioEncoder * encoder, GstBuffer * inbuf)
         info->channels, info->position, ffmpegaudenc->ffmpeg_layout);
   }
 
-  ret = gst_ffmpegaudenc_encode_audio (ffmpegaudenc, inbuf, &have_data);
+  ret = gst_ffmpegaudenc_send_frame (ffmpegaudenc, inbuf);
 
   if (ret != GST_FLOW_OK)
-    goto push_failed;
+    goto send_frame_failed;
+
+  do {
+    ret = gst_ffmpegaudenc_receive_packet (ffmpegaudenc, &got_packet);
+  } while (got_packet);
 
   return GST_FLOW_OK;
 
@@ -690,9 +659,9 @@ not_negotiated:
     gst_buffer_unref (inbuf);
     return GST_FLOW_NOT_NEGOTIATED;
   }
-push_failed:
+send_frame_failed:
   {
-    GST_DEBUG_OBJECT (ffmpegaudenc, "Failed to push buffer %d (%s)", ret,
+    GST_DEBUG_OBJECT (ffmpegaudenc, "Failed to send frame %d (%s)", ret,
         gst_flow_get_name (ret));
     return ret;
   }
@@ -704,55 +673,34 @@ gst_ffmpegaudenc_set_property (GObject * object,
 {
   GstFFMpegAudEnc *ffmpegaudenc;
 
-  /* Get a pointer of the right type. */
   ffmpegaudenc = (GstFFMpegAudEnc *) (object);
 
   if (ffmpegaudenc->opened) {
     GST_WARNING_OBJECT (ffmpegaudenc,
-        "Can't change properties once decoder is setup !");
+        "Can't change properties once encoder is setup !");
     return;
   }
 
-  /* Check the argument id to see which argument we're setting. */
   switch (prop_id) {
-    case PROP_BIT_RATE:
-      ffmpegaudenc->bitrate = g_value_get_int (value);
-      break;
-    case PROP_RTP_PAYLOAD_SIZE:
-      ffmpegaudenc->rtp_payload_size = g_value_get_int (value);
-      break;
-    case PROP_COMPLIANCE:
-      ffmpegaudenc->compliance = g_value_get_enum (value);
-      break;
     default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      if (!gst_ffmpeg_cfg_set_property (ffmpegaudenc->refcontext, value, pspec))
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
   }
 }
 
-/* The set function is simply the inverse of the get fuction. */
 static void
 gst_ffmpegaudenc_get_property (GObject * object,
     guint prop_id, GValue * value, GParamSpec * pspec)
 {
   GstFFMpegAudEnc *ffmpegaudenc;
 
-  /* It's not null if we got it, but it might not be ours */
   ffmpegaudenc = (GstFFMpegAudEnc *) (object);
 
   switch (prop_id) {
-    case PROP_BIT_RATE:
-      g_value_set_int (value, ffmpegaudenc->bitrate);
-      break;
-      break;
-    case PROP_RTP_PAYLOAD_SIZE:
-      g_value_set_int (value, ffmpegaudenc->rtp_payload_size);
-      break;
-    case PROP_COMPLIANCE:
-      g_value_set_enum (value, ffmpegaudenc->compliance);
-      break;
     default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      if (!gst_ffmpeg_cfg_get_property (ffmpegaudenc->refcontext, value, pspec))
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
   }
 }
@@ -773,32 +721,26 @@ gst_ffmpegaudenc_register (GstPlugin * plugin)
   };
   GType type;
   AVCodec *in_plugin;
+  void *i = 0;
 
 
   GST_LOG ("Registering encoders");
 
-  in_plugin = av_codec_next (NULL);
-  while (in_plugin) {
+  while ((in_plugin = (AVCodec *) av_codec_iterate (&i))) {
     gchar *type_name;
     guint rank;
 
     /* Skip non-AV codecs */
     if (in_plugin->type != AVMEDIA_TYPE_AUDIO)
-      goto next;
+      continue;
 
     /* no quasi codecs, please */
     if (in_plugin->id == AV_CODEC_ID_PCM_S16LE_PLANAR ||
         (in_plugin->id >= AV_CODEC_ID_PCM_S16LE &&
             in_plugin->id <= AV_CODEC_ID_PCM_BLURAY) ||
         (in_plugin->id >= AV_CODEC_ID_PCM_S8_PLANAR &&
-#if AV_VERSION_INT (LIBAVCODEC_VERSION_MAJOR, LIBAVCODEC_VERSION_MINOR, LIBAVCODEC_VERSION_MICRO) >= AV_VERSION_INT (57,69,0)
             in_plugin->id <= AV_CODEC_ID_PCM_F24LE)) {
-#elif AV_VERSION_INT (LIBAVCODEC_VERSION_MAJOR, LIBAVCODEC_VERSION_MINOR, LIBAVCODEC_VERSION_MICRO) >= AV_VERSION_INT (57,54,0)
-            in_plugin->id <= AV_CODEC_ID_PCM_S64BE)) {
-#else
-            in_plugin->id <= AV_CODEC_ID_PCM_S16BE_PLANAR)) {
-#endif
-      goto next;
+      continue;
     }
 
     /* No encoders depending on external libraries (we don't build them, but
@@ -808,12 +750,12 @@ gst_ffmpegaudenc_register (GstPlugin * plugin)
       GST_DEBUG
           ("Not using external library encoder %s. Use the gstreamer-native ones instead.",
           in_plugin->name);
-      goto next;
+      continue;
     }
 
     /* only encoders */
     if (!av_codec_is_encoder (in_plugin)) {
-      goto next;
+      continue;
     }
 
     /* FIXME : We should have a method to know cheaply whether we have a mapping
@@ -825,7 +767,7 @@ gst_ffmpegaudenc_register (GstPlugin * plugin)
     if (!strcmp (in_plugin->name, "vorbis")
         || !strcmp (in_plugin->name, "flac")) {
       GST_LOG ("Ignoring encoder %s", in_plugin->name);
-      goto next;
+      continue;
     }
 
     /* construct the type */
@@ -867,9 +809,6 @@ gst_ffmpegaudenc_register (GstPlugin * plugin)
     }
 
     g_free (type_name);
-
-  next:
-    in_plugin = av_codec_next (in_plugin);
   }
 
   GST_LOG ("Finished registering encoders");
