@@ -26,6 +26,7 @@
 
 #include <libavcodec/avcodec.h>
 #include <libavutil/stereo3d.h>
+#include <libavutil/mastering_display_metadata.h>
 
 #include "gstav.h"
 #include "gstavcodecmap.h"
@@ -45,6 +46,7 @@ GST_DEBUG_CATEGORY_STATIC (GST_CAT_PERFORMANCE);
 #define REQUIRED_POOL_MAX_BUFFERS       32
 #define DEFAULT_STRIDE_ALIGN            31
 #define DEFAULT_ALLOC_PARAM             { 0, DEFAULT_STRIDE_ALIGN, 0, 0, }
+#define DEFAULT_THREAD_TYPE             0
 
 enum
 {
@@ -55,6 +57,7 @@ enum
   PROP_DEBUG_MV,
   PROP_MAX_THREADS,
   PROP_OUTPUT_CORRUPT,
+  PROP_THREAD_TYPE,
   PROP_LAST
 };
 
@@ -143,6 +146,27 @@ gst_ffmpegviddec_skipframe_get_type (void)
   return ffmpegdec_skipframe_type;
 }
 
+static const GFlagsValue ffmpegdec_thread_types[] = {
+  {0x0, "Auto", "auto"},
+  {0x1, "Frame", "frame"},
+  {0x2, "Slice", "slice"},
+  {0, NULL, NULL},
+};
+
+#define GST_FFMPEGVIDDEC_TYPE_THREAD_TYPE (gst_ffmpegviddec_thread_type_get_type())
+static GType
+gst_ffmpegviddec_thread_type_get_type (void)
+{
+  static GType ffmpegdec_thread_type_type = 0;
+
+  if (!ffmpegdec_thread_type_type) {
+    ffmpegdec_thread_type_type =
+        g_flags_register_static ("GstLibAVVidDecThreadType",
+        ffmpegdec_thread_types);
+  }
+  return ffmpegdec_thread_type_type;
+}
+
 static void
 gst_ffmpegviddec_base_init (GstFFMpegVidDecClass * klass)
 {
@@ -239,6 +263,11 @@ gst_ffmpegviddec_class_init (GstFFMpegVidDecClass * klass)
             "Maximum number of worker threads to spawn. (0 = auto)",
             0, G_MAXINT, DEFAULT_MAX_THREADS,
             G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+    g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_THREAD_TYPE,
+        g_param_spec_flags ("thread-type", "Thread type",
+            "Multithreading methods to use",
+            GST_FFMPEGVIDDEC_TYPE_THREAD_TYPE,
+            DEFAULT_THREAD_TYPE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   }
 
   viddec_class->set_format = gst_ffmpegviddec_set_format;
@@ -252,6 +281,10 @@ gst_ffmpegviddec_class_init (GstFFMpegVidDecClass * klass)
   viddec_class->propose_allocation = gst_ffmpegviddec_propose_allocation;
 
   GST_DEBUG_CATEGORY_GET (GST_CAT_PERFORMANCE, "GST_PERFORMANCE");
+
+  gst_type_mark_as_plugin_api (GST_FFMPEGVIDDEC_TYPE_LOWRES, 0);
+  gst_type_mark_as_plugin_api (GST_FFMPEGVIDDEC_TYPE_SKIPFRAME, 0);
+  gst_type_mark_as_plugin_api (GST_FFMPEGVIDDEC_TYPE_THREAD_TYPE, 0);
 }
 
 static void
@@ -270,6 +303,7 @@ gst_ffmpegviddec_init (GstFFMpegVidDec * ffmpegdec)
   ffmpegdec->debug_mv = DEFAULT_DEBUG_MV;
   ffmpegdec->max_threads = DEFAULT_MAX_THREADS;
   ffmpegdec->output_corrupt = DEFAULT_OUTPUT_CORRUPT;
+  ffmpegdec->thread_type = DEFAULT_THREAD_TYPE;
 
   GST_PAD_SET_ACCEPT_TEMPLATE (GST_VIDEO_DECODER_SINK_PAD (ffmpegdec));
   gst_video_decoder_set_use_default_pad_acceptcaps (GST_VIDEO_DECODER_CAST
@@ -488,18 +522,14 @@ gst_ffmpegviddec_set_format (GstVideoDecoder * decoder,
    * supports it) */
   ffmpegdec->context->debug_mv = ffmpegdec->debug_mv;
 
-  {
+  if (ffmpegdec->thread_type) {
+    GST_DEBUG_OBJECT (ffmpegdec, "Use requested thread type 0x%x",
+        ffmpegdec->thread_type);
+    ffmpegdec->context->thread_type = ffmpegdec->thread_type;
+  } else {
     GstQuery *query;
     gboolean is_live;
 
-    if (ffmpegdec->max_threads == 0) {
-      if (!(oclass->in_plugin->capabilities & AV_CODEC_CAP_AUTO_THREADS))
-        ffmpegdec->context->thread_count = gst_ffmpeg_auto_max_threads ();
-      else
-        ffmpegdec->context->thread_count = 0;
-    } else
-      ffmpegdec->context->thread_count = ffmpegdec->max_threads;
-
     query = gst_query_new_latency ();
     is_live = FALSE;
     /* Check if upstream is live. If it isn't we can enable frame based
@@ -515,6 +545,18 @@ gst_ffmpegviddec_set_format (GstVideoDecoder * decoder,
       ffmpegdec->context->thread_type = FF_THREAD_SLICE | FF_THREAD_FRAME;
   }
 
+  if (ffmpegdec->max_threads == 0) {
+    /* When thread type is FF_THREAD_FRAME, extra latency is introduced equal
+     * to one frame per thread. We thus need to calculate the thread count ourselves */
+    if ((!(oclass->in_plugin->capabilities & AV_CODEC_CAP_AUTO_THREADS)) ||
+        (ffmpegdec->context->thread_type & FF_THREAD_FRAME))
+      ffmpegdec->context->thread_count =
+          MIN (gst_ffmpeg_auto_max_threads (), 16);
+    else
+      ffmpegdec->context->thread_count = 0;
+  } else
+    ffmpegdec->context->thread_count = ffmpegdec->max_threads;
+
   /* open codec - we don't select an output pix_fmt yet,
    * simply because we don't know! We only get it
    * during playback... */
@@ -530,6 +572,12 @@ gst_ffmpegviddec_set_format (GstVideoDecoder * decoder,
     latency = gst_util_uint64_scale_ceil (
         (ffmpegdec->context->has_b_frames) * GST_SECOND, info->fps_d,
         info->fps_n);
+
+    if (ffmpegdec->context->thread_type & FF_THREAD_FRAME) {
+      latency +=
+          gst_util_uint64_scale_ceil (ffmpegdec->context->thread_count *
+          GST_SECOND, info->fps_d, info->fps_n);
+    }
   }
 
   ret = TRUE;
@@ -1079,6 +1127,51 @@ stereo_av_to_gst (enum AVStereo3DType type)
   return GST_VIDEO_MULTIVIEW_MODE_NONE;
 }
 
+static gboolean
+mastering_display_metadata_av_to_gst (AVMasteringDisplayMetadata * av,
+    GstVideoMasteringDisplayInfo * gst)
+{
+  const guint64 chroma_scale = 50000;
+  const guint64 luma_scale = 10000;
+  gint i;
+
+  /* Use only complete mastering meta */
+  if (!av->has_primaries || !av->has_luminance)
+    return FALSE;
+
+  for (i = 0; i < G_N_ELEMENTS (gst->display_primaries); i++) {
+    gst->display_primaries[i].x = (guint16) gst_util_uint64_scale (chroma_scale,
+        av->display_primaries[i][0].num, av->display_primaries[i][0].den);
+    gst->display_primaries[i].y = (guint16) gst_util_uint64_scale (chroma_scale,
+        av->display_primaries[i][1].num, av->display_primaries[i][1].den);
+  }
+
+  gst->white_point.x = (guint16) gst_util_uint64_scale (chroma_scale,
+      av->white_point[0].num, av->white_point[0].den);
+  gst->white_point.y = (guint16) gst_util_uint64_scale (chroma_scale,
+      av->white_point[1].num, av->white_point[1].den);
+
+
+  gst->max_display_mastering_luminance =
+      (guint32) gst_util_uint64_scale (luma_scale,
+      av->max_luminance.num, av->max_luminance.den);
+  gst->min_display_mastering_luminance =
+      (guint32) gst_util_uint64_scale (luma_scale,
+      av->min_luminance.num, av->min_luminance.den);
+
+  return TRUE;
+}
+
+static gboolean
+content_light_metadata_av_to_gst (AVContentLightMetadata * av,
+    GstVideoContentLightLevel * gst)
+{
+  gst->max_content_light_level = av->MaxCLL;
+  gst->max_frame_average_light_level = av->MaxFALL;
+
+  return TRUE;
+}
+
 static gboolean
 gst_ffmpegviddec_negotiate (GstFFMpegVidDec * ffmpegdec,
     AVCodecContext * context, AVFrame * picture)
@@ -1150,100 +1243,30 @@ gst_ffmpegviddec_negotiate (GstFFMpegVidDec * ffmpegdec,
 
   if (!gst_structure_has_field (in_s, "colorimetry")
       || in_info->colorimetry.primaries == GST_VIDEO_COLOR_PRIMARIES_UNKNOWN) {
-    switch (context->color_primaries) {
-      case AVCOL_PRI_BT709:
-        out_info->colorimetry.primaries = GST_VIDEO_COLOR_PRIMARIES_BT709;
-        break;
-      case AVCOL_PRI_BT470M:
-        out_info->colorimetry.primaries = GST_VIDEO_COLOR_PRIMARIES_BT470M;
-        break;
-      case AVCOL_PRI_BT470BG:
-        out_info->colorimetry.primaries = GST_VIDEO_COLOR_PRIMARIES_BT470BG;
-        break;
-      case AVCOL_PRI_SMPTE170M:
-        out_info->colorimetry.primaries = GST_VIDEO_COLOR_PRIMARIES_SMPTE170M;
-        break;
-      case AVCOL_PRI_SMPTE240M:
-        out_info->colorimetry.primaries = GST_VIDEO_COLOR_PRIMARIES_SMPTE240M;
-        break;
-      case AVCOL_PRI_FILM:
-        out_info->colorimetry.primaries = GST_VIDEO_COLOR_PRIMARIES_FILM;
-        break;
-      case AVCOL_PRI_BT2020:
-        out_info->colorimetry.primaries = GST_VIDEO_COLOR_PRIMARIES_BT2020;
-        break;
-      default:
-        break;
-    }
+    out_info->colorimetry.primaries =
+        gst_video_color_primaries_from_iso (context->color_primaries);
   }
 
   if (!gst_structure_has_field (in_s, "colorimetry")
       || in_info->colorimetry.transfer == GST_VIDEO_TRANSFER_UNKNOWN) {
-    switch (context->color_trc) {
-      case AVCOL_TRC_BT2020_10:
-      case AVCOL_TRC_BT709:
-      case AVCOL_TRC_SMPTE170M:
-        out_info->colorimetry.transfer = GST_VIDEO_TRANSFER_BT709;
-        break;
-      case AVCOL_TRC_GAMMA22:
-        out_info->colorimetry.transfer = GST_VIDEO_TRANSFER_GAMMA22;
-        break;
-      case AVCOL_TRC_GAMMA28:
-        out_info->colorimetry.transfer = GST_VIDEO_TRANSFER_GAMMA28;
-        break;
-      case AVCOL_TRC_SMPTE240M:
-        out_info->colorimetry.transfer = GST_VIDEO_TRANSFER_SMPTE240M;
-        break;
-      case AVCOL_TRC_LINEAR:
-        out_info->colorimetry.transfer = GST_VIDEO_TRANSFER_GAMMA10;
-        break;
-      case AVCOL_TRC_LOG:
-        out_info->colorimetry.transfer = GST_VIDEO_TRANSFER_LOG100;
-        break;
-      case AVCOL_TRC_LOG_SQRT:
-        out_info->colorimetry.transfer = GST_VIDEO_TRANSFER_LOG316;
-        break;
-      case AVCOL_TRC_BT2020_12:
-        out_info->colorimetry.transfer = GST_VIDEO_TRANSFER_BT2020_12;
-        break;
-      default:
-        break;
-    }
+    out_info->colorimetry.transfer =
+        gst_video_transfer_function_from_iso (context->color_trc);
   }
 
   if (!gst_structure_has_field (in_s, "colorimetry")
       || in_info->colorimetry.matrix == GST_VIDEO_COLOR_MATRIX_UNKNOWN) {
-    switch (context->colorspace) {
-      case AVCOL_SPC_RGB:
-        out_info->colorimetry.matrix = GST_VIDEO_COLOR_MATRIX_RGB;
-        break;
-      case AVCOL_SPC_BT709:
-        out_info->colorimetry.matrix = GST_VIDEO_COLOR_MATRIX_BT709;
-        break;
-      case AVCOL_SPC_FCC:
-        out_info->colorimetry.matrix = GST_VIDEO_COLOR_MATRIX_FCC;
-        break;
-      case AVCOL_SPC_BT470BG:
-      case AVCOL_SPC_SMPTE170M:
-        out_info->colorimetry.matrix = GST_VIDEO_COLOR_MATRIX_BT601;
-        break;
-      case AVCOL_SPC_SMPTE240M:
-        out_info->colorimetry.matrix = GST_VIDEO_COLOR_MATRIX_SMPTE240M;
-        break;
-      case AVCOL_SPC_BT2020_NCL:
-        out_info->colorimetry.matrix = GST_VIDEO_COLOR_MATRIX_BT2020;
-        break;
-      default:
-        break;
-    }
+    out_info->colorimetry.matrix =
+        gst_video_color_matrix_from_iso (context->colorspace);
   }
 
   if (!gst_structure_has_field (in_s, "colorimetry")
       || in_info->colorimetry.range == GST_VIDEO_COLOR_RANGE_UNKNOWN) {
     if (context->color_range == AVCOL_RANGE_JPEG) {
       out_info->colorimetry.range = GST_VIDEO_COLOR_RANGE_0_255;
-    } else {
+    } else if (context->color_range == AVCOL_RANGE_MPEG) {
       out_info->colorimetry.range = GST_VIDEO_COLOR_RANGE_16_235;
+    } else {
+      out_info->colorimetry.range = GST_VIDEO_COLOR_RANGE_UNKNOWN;
     }
   }
 
@@ -1280,6 +1303,61 @@ gst_ffmpegviddec_negotiate (GstFFMpegVidDec * ffmpegdec,
   GST_VIDEO_INFO_MULTIVIEW_MODE (out_info) = ffmpegdec->cur_multiview_mode;
   GST_VIDEO_INFO_MULTIVIEW_FLAGS (out_info) = ffmpegdec->cur_multiview_flags;
 
+  /* To passing HDR information to caps directly */
+  if (output_state->caps == NULL) {
+    output_state->caps = gst_video_info_to_caps (out_info);
+  } else {
+    output_state->caps = gst_caps_make_writable (output_state->caps);
+  }
+
+  if (!gst_structure_has_field (in_s, "mastering-display-info")) {
+    AVFrameSideData *sd = av_frame_get_side_data (picture,
+        AV_FRAME_DATA_MASTERING_DISPLAY_METADATA);
+    GstVideoMasteringDisplayInfo minfo;
+
+    if (sd
+        && mastering_display_metadata_av_to_gst ((AVMasteringDisplayMetadata *)
+            sd->data, &minfo)) {
+      GST_LOG_OBJECT (ffmpegdec, "update mastering display info: "
+          "Red(%u, %u) "
+          "Green(%u, %u) "
+          "Blue(%u, %u) "
+          "White(%u, %u) "
+          "max_luminance(%u) "
+          "min_luminance(%u) ",
+          minfo.display_primaries[0].x, minfo.display_primaries[0].y,
+          minfo.display_primaries[1].x, minfo.display_primaries[1].y,
+          minfo.display_primaries[2].x, minfo.display_primaries[2].y,
+          minfo.white_point.x, minfo.white_point.y,
+          minfo.max_display_mastering_luminance,
+          minfo.min_display_mastering_luminance);
+
+      if (!gst_video_mastering_display_info_add_to_caps (&minfo,
+              output_state->caps)) {
+        GST_WARNING_OBJECT (ffmpegdec,
+            "Couldn't set mastering display info to caps");
+      }
+    }
+  }
+
+  if (!gst_structure_has_field (in_s, "content-light-level")) {
+    AVFrameSideData *sd = av_frame_get_side_data (picture,
+        AV_FRAME_DATA_CONTENT_LIGHT_LEVEL);
+    GstVideoContentLightLevel cll;
+
+    if (sd && content_light_metadata_av_to_gst ((AVContentLightMetadata *)
+            sd->data, &cll)) {
+      GST_LOG_OBJECT (ffmpegdec, "update content light level: "
+          "maxCLL:(%u), maxFALL:(%u)", cll.max_content_light_level,
+          cll.max_frame_average_light_level);
+
+      if (!gst_video_content_light_level_add_to_caps (&cll, output_state->caps)) {
+        GST_WARNING_OBJECT (ffmpegdec,
+            "Couldn't set content light level to caps");
+      }
+    }
+  }
+
   if (!gst_video_decoder_negotiate (GST_VIDEO_DECODER (ffmpegdec)))
     goto negotiate_failed;
 
@@ -1288,6 +1366,11 @@ gst_ffmpegviddec_negotiate (GstFFMpegVidDec * ffmpegdec,
     latency =
         gst_util_uint64_scale_ceil (ffmpegdec->context->has_b_frames *
         GST_SECOND, fps_d, fps_n);
+    if (ffmpegdec->context->thread_type & FF_THREAD_FRAME) {
+      latency +=
+          gst_util_uint64_scale_ceil (ffmpegdec->context->thread_count *
+          GST_SECOND, fps_d, fps_n);
+    }
     gst_video_decoder_set_latency (GST_VIDEO_DECODER (ffmpegdec), latency,
         latency);
   }
@@ -1608,26 +1691,14 @@ gst_ffmpegviddec_video_frame (GstFFMpegVidDec * ffmpegdec,
     AVFrameSideData *side_data =
         av_frame_get_side_data (ffmpegdec->picture, AV_FRAME_DATA_A53_CC);
     if (side_data) {
-      GstVideoCaptionMeta *cc_meta = NULL;
-      gpointer iter = NULL;
-      gboolean found_708_raw_meta = FALSE;
-
       GST_LOG_OBJECT (ffmpegdec,
           "Found CC side data of type AV_FRAME_DATA_A53_CC, size %d",
           side_data->size);
       GST_MEMDUMP ("A53 CC", side_data->data, side_data->size);
 
-      while ((cc_meta = (GstVideoCaptionMeta *)
-              gst_buffer_iterate_meta_filtered (out_frame->input_buffer, &iter,
-                  GST_VIDEO_CAPTION_META_API_TYPE))) {
-        if (cc_meta->caption_type != GST_VIDEO_CAPTION_TYPE_CEA708_RAW)
-          continue;
-        found_708_raw_meta = TRUE;
-        break;
-      }
-
-      /* do not add CEA 708 caption meta if it already exists */
-      if (!found_708_raw_meta) {
+      /* do not add closed caption meta if it already exists */
+      if (!gst_buffer_get_meta (out_frame->input_buffer,
+              GST_VIDEO_CAPTION_META_API_TYPE)) {
         out_frame->output_buffer =
             gst_buffer_make_writable (out_frame->output_buffer);
         gst_buffer_add_video_caption_meta (out_frame->output_buffer,
@@ -1635,7 +1706,7 @@ gst_ffmpegviddec_video_frame (GstFFMpegVidDec * ffmpegdec,
             side_data->size);
       } else {
         GST_LOG_OBJECT (ffmpegdec,
-            "CEA 708 caption meta already exists: will not add new caption meta");
+            "Closed caption meta already exists: will not add new caption meta");
       }
     }
   }
@@ -1739,28 +1810,19 @@ static GstFlowReturn
 gst_ffmpegviddec_drain (GstVideoDecoder * decoder)
 {
   GstFFMpegVidDec *ffmpegdec = (GstFFMpegVidDec *) decoder;
-  GstFFMpegVidDecClass *oclass;
+  GstFlowReturn ret;
+  gboolean got_frame = FALSE;
 
   if (!ffmpegdec->opened)
     return GST_FLOW_OK;
 
-  oclass = (GstFFMpegVidDecClass *) (G_OBJECT_GET_CLASS (ffmpegdec));
+  if (avcodec_send_packet (ffmpegdec->context, NULL))
+    goto send_packet_failed;
 
-  if (oclass->in_plugin->capabilities & AV_CODEC_CAP_DELAY) {
-    GstFlowReturn ret;
-    gboolean got_frame = FALSE;
-
-    GST_LOG_OBJECT (ffmpegdec,
-        "codec has delay capabilities, calling until ffmpeg has drained everything");
-
-    if (avcodec_send_packet (ffmpegdec->context, NULL))
-      goto send_packet_failed;
-
-    do {
-      got_frame = gst_ffmpegviddec_frame (ffmpegdec, NULL, &ret);
-    } while (got_frame && ret == GST_FLOW_OK);
-    avcodec_flush_buffers (ffmpegdec->context);
-  }
+  do {
+    got_frame = gst_ffmpegviddec_frame (ffmpegdec, NULL, &ret);
+  } while (got_frame && ret == GST_FLOW_OK);
+  avcodec_flush_buffers (ffmpegdec->context);
 
 done:
   return GST_FLOW_OK;
@@ -2154,6 +2216,9 @@ gst_ffmpegviddec_set_property (GObject * object,
     case PROP_OUTPUT_CORRUPT:
       ffmpegdec->output_corrupt = g_value_get_boolean (value);
       break;
+    case PROP_THREAD_TYPE:
+      ffmpegdec->thread_type = g_value_get_flags (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -2185,6 +2250,9 @@ gst_ffmpegviddec_get_property (GObject * object,
     case PROP_OUTPUT_CORRUPT:
       g_value_set_boolean (value, ffmpegdec->output_corrupt);
       break;
+    case PROP_THREAD_TYPE:
+      g_value_set_flags (value, ffmpegdec->thread_type);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -2251,6 +2319,22 @@ gst_ffmpegviddec_register (GstPlugin * plugin)
       continue;
     }
 
+    /* Skip hardware or hybrid (hardware with software fallback) */
+    if ((in_plugin->capabilities & AV_CODEC_CAP_HARDWARE) ==
+        AV_CODEC_CAP_HARDWARE) {
+      GST_DEBUG
+          ("Ignoring hardware decoder %s. We can't handle this outside of ffmpeg",
+          in_plugin->name);
+      continue;
+    }
+
+    if ((in_plugin->capabilities & AV_CODEC_CAP_HYBRID) == AV_CODEC_CAP_HYBRID) {
+      GST_DEBUG
+          ("Ignoring hybrid decoder %s. We can't handle this outside of ffmpeg",
+          in_plugin->name);
+      continue;
+    }
+
     /* No vdpau plugins until we can figure out how to properly use them
      * outside of ffmpeg. */
     if (g_str_has_suffix (in_plugin->name, "_vdpau")) {
@@ -2281,20 +2365,6 @@ gst_ffmpegviddec_register (GstPlugin * plugin)
       continue;
     }
 
-    if (g_str_has_suffix (in_plugin->name, "_cuvid")) {
-      GST_DEBUG
-          ("Ignoring CUVID decoder %s. We can't handle this outside of ffmpeg",
-          in_plugin->name);
-      continue;
-    }
-
-    if (g_str_has_suffix (in_plugin->name, "_v4l2m2m")) {
-      GST_DEBUG
-          ("Ignoring V4L2 mem-to-mem decoder %s. We can't handle this outside of ffmpeg",
-          in_plugin->name);
-      continue;
-    }
-
     GST_DEBUG ("Trying plugin %s [%s]", in_plugin->name, in_plugin->long_name);
 
     /* no codecs for which we're GUARANTEED to have better alternatives */
@@ -2302,6 +2372,7 @@ gst_ffmpegviddec_register (GstPlugin * plugin)
     /* MP1 : Use MP3 for decoding */
     /* MP2 : Use MP3 for decoding */
     /* Theora: Use libtheora based theoradec */
+    /* CDG: use cdgdec */
     if (!strcmp (in_plugin->name, "theora") ||
         !strcmp (in_plugin->name, "mpeg1video") ||
         strstr (in_plugin->name, "crystalhd") != NULL ||
@@ -2309,7 +2380,8 @@ gst_ffmpegviddec_register (GstPlugin * plugin)
         !strcmp (in_plugin->name, "srt") ||
         !strcmp (in_plugin->name, "pgssub") ||
         !strcmp (in_plugin->name, "dvdsub") ||
-        !strcmp (in_plugin->name, "dvbsub")) {
+        !strcmp (in_plugin->name, "dvbsub") ||
+        !strcmp (in_plugin->name, "cdgraphics")) {
       GST_LOG ("Ignoring decoder %s", in_plugin->name);
       continue;
     }
