@@ -196,6 +196,8 @@ gst_ffmpegvidenc_class_init (GstFFMpegVidEncClass * klass)
   venc_class->flush = gst_ffmpegvidenc_flush;
 
   gobject_class->finalize = gst_ffmpegvidenc_finalize;
+
+  gst_type_mark_as_plugin_api (GST_TYPE_FFMPEG_PASS, 0);
 }
 
 static void
@@ -221,6 +223,7 @@ gst_ffmpegvidenc_finalize (GObject * object)
   /* clean up remaining allocated data */
   av_frame_free (&ffmpegenc->picture);
   gst_ffmpeg_avcodec_close (ffmpegenc->context);
+  gst_ffmpeg_avcodec_close (ffmpegenc->refcontext);
   av_free (ffmpegenc->context);
   av_free (ffmpegenc->refcontext);
 
@@ -500,6 +503,9 @@ static enum AVStereo3DType
 stereo_gst_to_av (GstVideoMultiviewMode mview_mode)
 {
   switch (mview_mode) {
+    case GST_VIDEO_MULTIVIEW_MODE_MONO:
+      /* Video is not stereoscopic (and metadata has to be there). */
+      return AV_STEREO3D_2D;
     case GST_VIDEO_MULTIVIEW_MODE_SIDE_BY_SIDE:
       return AV_STEREO3D_SIDEBYSIDE;
     case GST_VIDEO_MULTIVIEW_MODE_TOP_BOTTOM:
@@ -681,6 +687,15 @@ gst_ffmpegvidenc_receive_packet (GstFFMpegVidEnc * ffmpegenc,
       GST_VIDEO_CODEC_FRAME_UNSET_SYNC_POINT (frame);
   }
 
+  frame->dts =
+      gst_ffmpeg_time_ff_to_gst (pkt->dts, ffmpegenc->context->time_base);
+  /* This will lose some precision compared to setting the PTS from the input
+   * buffer directly, but that way we're sure PTS and DTS are consistent, in
+   * particular DTS should always be <= PTS
+   */
+  frame->pts =
+      gst_ffmpeg_time_ff_to_gst (pkt->pts, ffmpegenc->context->time_base);
+
   ret = gst_video_encoder_finish_frame (GST_VIDEO_ENCODER (ffmpegenc), frame);
 
 done:
@@ -835,6 +850,8 @@ gst_ffmpegvidenc_start (GstVideoEncoder * encoder)
     return FALSE;
   }
 
+  gst_video_encoder_set_min_pts (encoder, GST_SECOND * 60 * 60 * 1000);
+
   return TRUE;
 }
 
@@ -920,30 +937,18 @@ gst_ffmpegvidenc_register (GstPlugin * plugin)
       continue;
     }
 
-    if (strstr (in_plugin->name, "vaapi")) {
+    /* Skip hardware or hybrid (hardware with software fallback) */
+    if ((in_plugin->capabilities & AV_CODEC_CAP_HARDWARE) ==
+        AV_CODEC_CAP_HARDWARE) {
       GST_DEBUG
-          ("Ignoring VAAPI encoder %s. We can't handle this outside of ffmpeg",
+          ("Ignoring hardware encoder %s. We can't handle this outside of ffmpeg",
           in_plugin->name);
       continue;
     }
 
-    if (strstr (in_plugin->name, "nvenc")) {
+    if ((in_plugin->capabilities & AV_CODEC_CAP_HYBRID) == AV_CODEC_CAP_HYBRID) {
       GST_DEBUG
-          ("Ignoring nvenc encoder %s. We can't handle this outside of ffmpeg",
-          in_plugin->name);
-      continue;
-    }
-
-    if (g_str_has_suffix (in_plugin->name, "_qsv")) {
-      GST_DEBUG
-          ("Ignoring qsv encoder %s. We can't handle this outside of ffmpeg",
-          in_plugin->name);
-      continue;
-    }
-
-    if (g_str_has_suffix (in_plugin->name, "_v4l2m2m")) {
-      GST_DEBUG
-          ("Ignoring V4L2 mem-to-mem encoder %s. We can't handle this outside of ffmpeg",
+          ("Ignoring hybrid encoder %s. We can't handle this outside of ffmpeg",
           in_plugin->name);
       continue;
     }
